# Generated from ./deployment.g4 by ANTLR 4.13.1
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,59,297,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,2,12,7,12,2,13,7,13,
        2,14,7,14,2,15,7,15,1,0,1,0,1,0,1,0,1,0,4,0,38,8,0,11,0,12,0,39,
        1,0,1,0,1,0,1,0,4,0,46,8,0,11,0,12,0,47,1,0,1,0,1,0,1,0,4,0,54,8,
        0,11,0,12,0,55,1,0,1,0,1,0,1,0,4,0,62,8,0,11,0,12,0,63,1,0,1,0,1,
        0,1,0,4,0,70,8,0,11,0,12,0,71,1,0,1,0,3,0,76,8,0,1,0,1,0,1,0,4,0,
        81,8,0,11,0,12,0,82,1,0,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
        1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
        1,1,1,1,1,1,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,
        1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,3,2,139,8,2,1,3,1,3,
        1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,
        1,3,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,
        3,4,175,8,4,1,4,1,4,1,5,1,5,1,5,1,5,1,5,1,5,1,5,1,5,1,5,1,5,1,5,
        3,5,190,8,5,1,5,3,5,193,8,5,1,6,1,6,3,6,197,8,6,1,7,1,7,1,7,1,7,
        1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,
        1,7,1,7,1,7,1,7,1,7,1,7,1,7,3,7,226,8,7,1,7,1,7,1,7,1,7,1,7,1,7,
        1,7,3,7,235,8,7,1,7,3,7,238,8,7,1,7,1,7,1,7,1,7,1,7,1,7,1,7,3,7,
        247,8,7,1,7,3,7,250,8,7,1,8,1,8,1,8,1,8,1,8,1,8,1,8,1,9,1,9,1,9,
        1,9,1,9,1,9,3,9,265,8,9,1,9,1,9,1,10,1,10,1,10,1,10,1,10,1,10,3,
        10,275,8,10,1,10,1,10,1,11,1,11,1,11,1,11,1,11,1,11,3,11,285,8,11,
        1,11,1,11,1,12,1,12,1,13,1,13,1,14,1,14,1,15,1,15,1,15,0,0,16,0,
        2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,0,4,1,0,39,43,1,0,44,46,
        1,0,47,50,1,0,51,52,300,0,32,1,0,0,0,2,87,1,0,0,0,4,114,1,0,0,0,
        6,140,1,0,0,0,8,159,1,0,0,0,10,178,1,0,0,0,12,196,1,0,0,0,14,198,
        1,0,0,0,16,251,1,0,0,0,18,258,1,0,0,0,20,268,1,0,0,0,22,278,1,0,
        0,0,24,288,1,0,0,0,26,290,1,0,0,0,28,292,1,0,0,0,30,294,1,0,0,0,
        32,33,5,1,0,0,33,34,5,2,0,0,34,35,5,3,0,0,35,37,5,2,0,0,36,38,3,
        2,1,0,37,36,1,0,0,0,38,39,1,0,0,0,39,37,1,0,0,0,39,40,1,0,0,0,40,
        41,1,0,0,0,41,42,5,4,0,0,42,43,5,5,0,0,43,45,5,2,0,0,44,46,3,4,2,
        0,45,44,1,0,0,0,46,47,1,0,0,0,47,45,1,0,0,0,47,48,1,0,0,0,48,49,
        1,0,0,0,49,50,5,4,0,0,50,51,5,6,0,0,51,53,5,2,0,0,52,54,3,6,3,0,
        53,52,1,0,0,0,54,55,1,0,0,0,55,53,1,0,0,0,55,56,1,0,0,0,56,57,1,
        0,0,0,57,58,5,4,0,0,58,59,5,7,0,0,59,61,5,2,0,0,60,62,3,8,4,0,61,
        60,1,0,0,0,62,63,1,0,0,0,63,61,1,0,0,0,63,64,1,0,0,0,64,65,1,0,0,
        0,65,75,5,4,0,0,66,67,5,8,0,0,67,69,5,2,0,0,68,70,3,10,5,0,69,68,
        1,0,0,0,70,71,1,0,0,0,71,69,1,0,0,0,71,72,1,0,0,0,72,73,1,0,0,0,
        73,74,5,4,0,0,74,76,1,0,0,0,75,66,1,0,0,0,75,76,1,0,0,0,76,77,1,
        0,0,0,77,78,5,9,0,0,78,80,5,2,0,0,79,81,3,12,6,0,80,79,1,0,0,0,81,
        82,1,0,0,0,82,80,1,0,0,0,82,83,1,0,0,0,83,84,1,0,0,0,84,85,5,4,0,
        0,85,86,5,4,0,0,86,1,1,0,0,0,87,88,5,10,0,0,88,89,5,11,0,0,89,90,
        5,12,0,0,90,91,5,57,0,0,91,92,5,13,0,0,92,93,5,14,0,0,93,94,5,12,
        0,0,94,95,5,59,0,0,95,96,5,13,0,0,96,97,5,15,0,0,97,98,5,12,0,0,
        98,99,5,56,0,0,99,100,5,13,0,0,100,101,5,16,0,0,101,102,5,12,0,0,
        102,103,5,56,0,0,103,104,5,17,0,0,104,105,5,13,0,0,105,106,5,18,
        0,0,106,107,5,12,0,0,107,108,5,56,0,0,108,109,5,19,0,0,109,110,5,
        13,0,0,110,111,5,20,0,0,111,112,5,12,0,0,112,113,5,59,0,0,113,3,
        1,0,0,0,114,115,5,10,0,0,115,116,5,11,0,0,116,117,5,12,0,0,117,118,
        5,57,0,0,118,119,5,13,0,0,119,120,5,15,0,0,120,121,5,12,0,0,121,
        122,5,56,0,0,122,123,5,13,0,0,123,124,5,21,0,0,124,125,5,12,0,0,
        125,126,5,56,0,0,126,127,5,13,0,0,127,128,5,22,0,0,128,129,5,12,
        0,0,129,130,3,24,12,0,130,131,5,13,0,0,131,132,5,23,0,0,132,133,
        5,12,0,0,133,138,3,26,13,0,134,135,5,13,0,0,135,136,5,24,0,0,136,
        137,5,12,0,0,137,139,5,57,0,0,138,134,1,0,0,0,138,139,1,0,0,0,139,
        5,1,0,0,0,140,141,5,10,0,0,141,142,5,11,0,0,142,143,5,12,0,0,143,
        144,5,57,0,0,144,145,5,13,0,0,145,146,5,24,0,0,146,147,5,12,0,0,
        147,148,5,57,0,0,148,149,5,13,0,0,149,150,5,25,0,0,150,151,5,12,
        0,0,151,152,5,56,0,0,152,153,5,17,0,0,153,154,5,13,0,0,154,155,5,
        26,0,0,155,156,5,12,0,0,156,157,5,56,0,0,157,158,5,19,0,0,158,7,
        1,0,0,0,159,160,5,10,0,0,160,161,5,11,0,0,161,162,5,12,0,0,162,163,
        5,57,0,0,163,164,5,13,0,0,164,165,5,27,0,0,165,166,5,12,0,0,166,
        167,5,56,0,0,167,168,5,13,0,0,168,169,5,6,0,0,169,170,5,12,0,0,170,
        171,5,28,0,0,171,174,5,57,0,0,172,173,5,13,0,0,173,175,5,57,0,0,
        174,172,1,0,0,0,174,175,1,0,0,0,175,176,1,0,0,0,176,177,5,29,0,0,
        177,9,1,0,0,0,178,179,5,10,0,0,179,180,5,11,0,0,180,181,5,12,0,0,
        181,192,5,58,0,0,182,183,5,13,0,0,183,184,5,30,0,0,184,185,5,12,
        0,0,185,186,5,28,0,0,186,189,5,58,0,0,187,188,5,13,0,0,188,190,5,
        58,0,0,189,187,1,0,0,0,189,190,1,0,0,0,190,191,1,0,0,0,191,193,5,
        29,0,0,192,182,1,0,0,0,192,193,1,0,0,0,193,11,1,0,0,0,194,197,3,
        16,8,0,195,197,3,14,7,0,196,194,1,0,0,0,196,195,1,0,0,0,197,13,1,
        0,0,0,198,199,5,10,0,0,199,200,5,31,0,0,200,201,5,11,0,0,201,202,
        5,12,0,0,202,203,5,57,0,0,203,204,5,13,0,0,204,205,5,32,0,0,205,
        206,5,12,0,0,206,207,5,56,0,0,207,208,5,13,0,0,208,209,5,33,0,0,
        209,210,5,12,0,0,210,211,3,28,14,0,211,212,5,13,0,0,212,213,5,34,
        0,0,213,214,5,12,0,0,214,215,5,59,0,0,215,216,5,13,0,0,216,217,3,
        18,9,0,217,218,5,13,0,0,218,219,3,20,10,0,219,220,5,13,0,0,220,225,
        3,22,11,0,221,222,5,13,0,0,222,223,5,35,0,0,223,224,5,12,0,0,224,
        226,3,30,15,0,225,221,1,0,0,0,225,226,1,0,0,0,226,237,1,0,0,0,227,
        228,5,13,0,0,228,229,5,36,0,0,229,230,5,12,0,0,230,231,5,28,0,0,
        231,234,5,57,0,0,232,233,5,13,0,0,233,235,5,57,0,0,234,232,1,0,0,
        0,234,235,1,0,0,0,235,236,1,0,0,0,236,238,5,29,0,0,237,227,1,0,0,
        0,237,238,1,0,0,0,238,249,1,0,0,0,239,240,5,13,0,0,240,241,5,37,
        0,0,241,242,5,12,0,0,242,243,5,28,0,0,243,246,5,57,0,0,244,245,5,
        13,0,0,245,247,5,57,0,0,246,244,1,0,0,0,246,247,1,0,0,0,247,248,
        1,0,0,0,248,250,5,29,0,0,249,239,1,0,0,0,249,250,1,0,0,0,250,15,
        1,0,0,0,251,252,5,38,0,0,252,253,5,2,0,0,253,254,5,11,0,0,254,255,
        5,12,0,0,255,256,5,57,0,0,256,257,5,4,0,0,257,17,1,0,0,0,258,259,
        5,5,0,0,259,260,5,12,0,0,260,261,5,28,0,0,261,264,5,57,0,0,262,263,
        5,13,0,0,263,265,5,57,0,0,264,262,1,0,0,0,264,265,1,0,0,0,265,266,
        1,0,0,0,266,267,5,29,0,0,267,19,1,0,0,0,268,269,5,7,0,0,269,270,
        5,12,0,0,270,271,5,28,0,0,271,274,5,57,0,0,272,273,5,13,0,0,273,
        275,5,57,0,0,274,272,1,0,0,0,274,275,1,0,0,0,275,276,1,0,0,0,276,
        277,5,29,0,0,277,21,1,0,0,0,278,279,5,8,0,0,279,280,5,12,0,0,280,
        281,5,28,0,0,281,284,5,58,0,0,282,283,5,13,0,0,283,285,5,58,0,0,
        284,282,1,0,0,0,284,285,1,0,0,0,285,286,1,0,0,0,286,287,5,29,0,0,
        287,23,1,0,0,0,288,289,7,0,0,0,289,25,1,0,0,0,290,291,7,1,0,0,291,
        27,1,0,0,0,292,293,7,2,0,0,293,29,1,0,0,0,294,295,7,3,0,0,295,31,
        1,0,0,0,20,39,47,55,63,71,75,82,138,174,189,192,196,225,234,237,
        246,249,264,274,284
    ]

class deploymentParser ( Parser ):

    grammarFileName = "deployment.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'Deployment model'", "'{'", "'applications'", 
                     "'}'", "'services'", "'containers'", "'deployments'", 
                     "'regions'", "'clusters'", "'->'", "'name'", "':'", 
                     "','", "'image'", "'port'", "'cpu_required'", "'m'", 
                     "'memory_required'", "'Mi'", "'domain_model'", "'target_port'", 
                     "'protocol'", "'type'", "'app_name'", "'cpu_limit'", 
                     "'memory_limit'", "'replicas'", "'['", "']'", "'zones'", 
                     "'public_cluster'", "'number_of_nodes'", "'provider'", 
                     "'config_file'", "'net_config'", "'networks'", "'subnetworks'", 
                     "'private_cluster'", "'HTTP'", "'HTTPS'", "'TCP'", 
                     "'UDP'", "'ALL'", "'lb'", "'ingress'", "'egress'", 
                     "'google'", "'aws'", "'azure'", "'other'", "'True'", 
                     "'False'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "WS", "ML_COMMENT", "SL_COMMENT", "INT", 
                      "ID", "ID_REG", "STRING" ]

    RULE_architecture = 0
    RULE_application = 1
    RULE_service = 2
    RULE_container = 3
    RULE_deployment = 4
    RULE_region = 5
    RULE_cluster = 6
    RULE_publicCluster = 7
    RULE_privateCluster = 8
    RULE_service_list = 9
    RULE_deployment_list = 10
    RULE_region_list = 11
    RULE_protocol = 12
    RULE_service_type = 13
    RULE_provider = 14
    RULE_boolean = 15

    ruleNames =  [ "architecture", "application", "service", "container", 
                   "deployment", "region", "cluster", "publicCluster", "privateCluster", 
                   "service_list", "deployment_list", "region_list", "protocol", 
                   "service_type", "provider", "boolean" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    T__21=22
    T__22=23
    T__23=24
    T__24=25
    T__25=26
    T__26=27
    T__27=28
    T__28=29
    T__29=30
    T__30=31
    T__31=32
    T__32=33
    T__33=34
    T__34=35
    T__35=36
    T__36=37
    T__37=38
    T__38=39
    T__39=40
    T__40=41
    T__41=42
    T__42=43
    T__43=44
    T__44=45
    T__45=46
    T__46=47
    T__47=48
    T__48=49
    T__49=50
    T__50=51
    T__51=52
    WS=53
    ML_COMMENT=54
    SL_COMMENT=55
    INT=56
    ID=57
    ID_REG=58
    STRING=59

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.1")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class ArchitectureContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def application(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.ApplicationContext)
            else:
                return self.getTypedRuleContext(deploymentParser.ApplicationContext,i)


        def service(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.ServiceContext)
            else:
                return self.getTypedRuleContext(deploymentParser.ServiceContext,i)


        def container(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.ContainerContext)
            else:
                return self.getTypedRuleContext(deploymentParser.ContainerContext,i)


        def deployment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.DeploymentContext)
            else:
                return self.getTypedRuleContext(deploymentParser.DeploymentContext,i)


        def cluster(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.ClusterContext)
            else:
                return self.getTypedRuleContext(deploymentParser.ClusterContext,i)


        def region(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(deploymentParser.RegionContext)
            else:
                return self.getTypedRuleContext(deploymentParser.RegionContext,i)


        def getRuleIndex(self):
            return deploymentParser.RULE_architecture

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitecture" ):
                listener.enterArchitecture(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitecture" ):
                listener.exitArchitecture(self)




    def architecture(self):

        localctx = deploymentParser.ArchitectureContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_architecture)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 32
            self.match(deploymentParser.T__0)
            self.state = 33
            self.match(deploymentParser.T__1)
            self.state = 34
            self.match(deploymentParser.T__2)
            self.state = 35
            self.match(deploymentParser.T__1)
            self.state = 37 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 36
                self.application()
                self.state = 39 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==10):
                    break

            self.state = 41
            self.match(deploymentParser.T__3)
            self.state = 42
            self.match(deploymentParser.T__4)
            self.state = 43
            self.match(deploymentParser.T__1)
            self.state = 45 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 44
                self.service()
                self.state = 47 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==10):
                    break

            self.state = 49
            self.match(deploymentParser.T__3)
            self.state = 50
            self.match(deploymentParser.T__5)
            self.state = 51
            self.match(deploymentParser.T__1)
            self.state = 53 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 52
                self.container()
                self.state = 55 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==10):
                    break

            self.state = 57
            self.match(deploymentParser.T__3)
            self.state = 58
            self.match(deploymentParser.T__6)
            self.state = 59
            self.match(deploymentParser.T__1)
            self.state = 61 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 60
                self.deployment()
                self.state = 63 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==10):
                    break

            self.state = 65
            self.match(deploymentParser.T__3)
            self.state = 75
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==8:
                self.state = 66
                self.match(deploymentParser.T__7)
                self.state = 67
                self.match(deploymentParser.T__1)
                self.state = 69 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while True:
                    self.state = 68
                    self.region()
                    self.state = 71 
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if not (_la==10):
                        break

                self.state = 73
                self.match(deploymentParser.T__3)


            self.state = 77
            self.match(deploymentParser.T__8)
            self.state = 78
            self.match(deploymentParser.T__1)
            self.state = 80 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 79
                self.cluster()
                self.state = 82 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==10 or _la==38):
                    break

            self.state = 84
            self.match(deploymentParser.T__3)
            self.state = 85
            self.match(deploymentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ApplicationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self):
            return self.getToken(deploymentParser.ID, 0)

        def STRING(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.STRING)
            else:
                return self.getToken(deploymentParser.STRING, i)

        def INT(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.INT)
            else:
                return self.getToken(deploymentParser.INT, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_application

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterApplication" ):
                listener.enterApplication(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitApplication" ):
                listener.exitApplication(self)




    def application(self):

        localctx = deploymentParser.ApplicationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_application)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 87
            self.match(deploymentParser.T__9)
            self.state = 88
            self.match(deploymentParser.T__10)
            self.state = 89
            self.match(deploymentParser.T__11)
            self.state = 90
            self.match(deploymentParser.ID)
            self.state = 91
            self.match(deploymentParser.T__12)
            self.state = 92
            self.match(deploymentParser.T__13)
            self.state = 93
            self.match(deploymentParser.T__11)
            self.state = 94
            self.match(deploymentParser.STRING)
            self.state = 95
            self.match(deploymentParser.T__12)
            self.state = 96
            self.match(deploymentParser.T__14)
            self.state = 97
            self.match(deploymentParser.T__11)
            self.state = 98
            self.match(deploymentParser.INT)
            self.state = 99
            self.match(deploymentParser.T__12)
            self.state = 100
            self.match(deploymentParser.T__15)
            self.state = 101
            self.match(deploymentParser.T__11)
            self.state = 102
            self.match(deploymentParser.INT)
            self.state = 103
            self.match(deploymentParser.T__16)
            self.state = 104
            self.match(deploymentParser.T__12)
            self.state = 105
            self.match(deploymentParser.T__17)
            self.state = 106
            self.match(deploymentParser.T__11)
            self.state = 107
            self.match(deploymentParser.INT)
            self.state = 108
            self.match(deploymentParser.T__18)
            self.state = 109
            self.match(deploymentParser.T__12)
            self.state = 110
            self.match(deploymentParser.T__19)
            self.state = 111
            self.match(deploymentParser.T__11)
            self.state = 112
            self.match(deploymentParser.STRING)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ServiceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.app = None # Token

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def INT(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.INT)
            else:
                return self.getToken(deploymentParser.INT, i)

        def protocol(self):
            return self.getTypedRuleContext(deploymentParser.ProtocolContext,0)


        def service_type(self):
            return self.getTypedRuleContext(deploymentParser.Service_typeContext,0)


        def getRuleIndex(self):
            return deploymentParser.RULE_service

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterService" ):
                listener.enterService(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitService" ):
                listener.exitService(self)




    def service(self):

        localctx = deploymentParser.ServiceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_service)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 114
            self.match(deploymentParser.T__9)
            self.state = 115
            self.match(deploymentParser.T__10)
            self.state = 116
            self.match(deploymentParser.T__11)
            self.state = 117
            self.match(deploymentParser.ID)
            self.state = 118
            self.match(deploymentParser.T__12)
            self.state = 119
            self.match(deploymentParser.T__14)
            self.state = 120
            self.match(deploymentParser.T__11)
            self.state = 121
            self.match(deploymentParser.INT)
            self.state = 122
            self.match(deploymentParser.T__12)
            self.state = 123
            self.match(deploymentParser.T__20)
            self.state = 124
            self.match(deploymentParser.T__11)
            self.state = 125
            self.match(deploymentParser.INT)
            self.state = 126
            self.match(deploymentParser.T__12)
            self.state = 127
            self.match(deploymentParser.T__21)
            self.state = 128
            self.match(deploymentParser.T__11)
            self.state = 129
            self.protocol()
            self.state = 130
            self.match(deploymentParser.T__12)
            self.state = 131
            self.match(deploymentParser.T__22)
            self.state = 132
            self.match(deploymentParser.T__11)
            self.state = 133
            self.service_type()
            self.state = 138
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 134
                self.match(deploymentParser.T__12)
                self.state = 135
                self.match(deploymentParser.T__23)
                self.state = 136
                self.match(deploymentParser.T__11)
                self.state = 137
                localctx.app = self.match(deploymentParser.ID)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ContainerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def INT(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.INT)
            else:
                return self.getToken(deploymentParser.INT, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_container

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterContainer" ):
                listener.enterContainer(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitContainer" ):
                listener.exitContainer(self)




    def container(self):

        localctx = deploymentParser.ContainerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_container)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 140
            self.match(deploymentParser.T__9)
            self.state = 141
            self.match(deploymentParser.T__10)
            self.state = 142
            self.match(deploymentParser.T__11)
            self.state = 143
            self.match(deploymentParser.ID)
            self.state = 144
            self.match(deploymentParser.T__12)
            self.state = 145
            self.match(deploymentParser.T__23)
            self.state = 146
            self.match(deploymentParser.T__11)
            self.state = 147
            self.match(deploymentParser.ID)
            self.state = 148
            self.match(deploymentParser.T__12)
            self.state = 149
            self.match(deploymentParser.T__24)
            self.state = 150
            self.match(deploymentParser.T__11)
            self.state = 151
            self.match(deploymentParser.INT)
            self.state = 152
            self.match(deploymentParser.T__16)
            self.state = 153
            self.match(deploymentParser.T__12)
            self.state = 154
            self.match(deploymentParser.T__25)
            self.state = 155
            self.match(deploymentParser.T__11)
            self.state = 156
            self.match(deploymentParser.INT)
            self.state = 157
            self.match(deploymentParser.T__18)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DeploymentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def INT(self):
            return self.getToken(deploymentParser.INT, 0)

        def getRuleIndex(self):
            return deploymentParser.RULE_deployment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDeployment" ):
                listener.enterDeployment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDeployment" ):
                listener.exitDeployment(self)




    def deployment(self):

        localctx = deploymentParser.DeploymentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_deployment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 159
            self.match(deploymentParser.T__9)
            self.state = 160
            self.match(deploymentParser.T__10)
            self.state = 161
            self.match(deploymentParser.T__11)
            self.state = 162
            self.match(deploymentParser.ID)
            self.state = 163
            self.match(deploymentParser.T__12)
            self.state = 164
            self.match(deploymentParser.T__26)
            self.state = 165
            self.match(deploymentParser.T__11)
            self.state = 166
            self.match(deploymentParser.INT)
            self.state = 167
            self.match(deploymentParser.T__12)
            self.state = 168
            self.match(deploymentParser.T__5)
            self.state = 169
            self.match(deploymentParser.T__11)
            self.state = 170
            self.match(deploymentParser.T__27)
            self.state = 171
            self.match(deploymentParser.ID)
            self.state = 174
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 172
                self.match(deploymentParser.T__12)
                self.state = 173
                self.match(deploymentParser.ID)


            self.state = 176
            self.match(deploymentParser.T__28)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RegionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID_REG(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID_REG)
            else:
                return self.getToken(deploymentParser.ID_REG, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_region

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRegion" ):
                listener.enterRegion(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRegion" ):
                listener.exitRegion(self)




    def region(self):

        localctx = deploymentParser.RegionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_region)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 178
            self.match(deploymentParser.T__9)
            self.state = 179
            self.match(deploymentParser.T__10)
            self.state = 180
            self.match(deploymentParser.T__11)
            self.state = 181
            self.match(deploymentParser.ID_REG)
            self.state = 192
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 182
                self.match(deploymentParser.T__12)
                self.state = 183
                self.match(deploymentParser.T__29)
                self.state = 184
                self.match(deploymentParser.T__11)
                self.state = 185
                self.match(deploymentParser.T__27)
                self.state = 186
                self.match(deploymentParser.ID_REG)
                self.state = 189
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==13:
                    self.state = 187
                    self.match(deploymentParser.T__12)
                    self.state = 188
                    self.match(deploymentParser.ID_REG)


                self.state = 191
                self.match(deploymentParser.T__28)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ClusterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def privateCluster(self):
            return self.getTypedRuleContext(deploymentParser.PrivateClusterContext,0)


        def publicCluster(self):
            return self.getTypedRuleContext(deploymentParser.PublicClusterContext,0)


        def getRuleIndex(self):
            return deploymentParser.RULE_cluster

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCluster" ):
                listener.enterCluster(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCluster" ):
                listener.exitCluster(self)




    def cluster(self):

        localctx = deploymentParser.ClusterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_cluster)
        try:
            self.state = 196
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [38]:
                self.enterOuterAlt(localctx, 1)
                self.state = 194
                self.privateCluster()
                pass
            elif token in [10]:
                self.enterOuterAlt(localctx, 2)
                self.state = 195
                self.publicCluster()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PublicClusterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def INT(self):
            return self.getToken(deploymentParser.INT, 0)

        def provider(self):
            return self.getTypedRuleContext(deploymentParser.ProviderContext,0)


        def STRING(self):
            return self.getToken(deploymentParser.STRING, 0)

        def service_list(self):
            return self.getTypedRuleContext(deploymentParser.Service_listContext,0)


        def deployment_list(self):
            return self.getTypedRuleContext(deploymentParser.Deployment_listContext,0)


        def region_list(self):
            return self.getTypedRuleContext(deploymentParser.Region_listContext,0)


        def boolean(self):
            return self.getTypedRuleContext(deploymentParser.BooleanContext,0)


        def getRuleIndex(self):
            return deploymentParser.RULE_publicCluster

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPublicCluster" ):
                listener.enterPublicCluster(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPublicCluster" ):
                listener.exitPublicCluster(self)




    def publicCluster(self):

        localctx = deploymentParser.PublicClusterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_publicCluster)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 198
            self.match(deploymentParser.T__9)
            self.state = 199
            self.match(deploymentParser.T__30)
            self.state = 200
            self.match(deploymentParser.T__10)
            self.state = 201
            self.match(deploymentParser.T__11)
            self.state = 202
            self.match(deploymentParser.ID)
            self.state = 203
            self.match(deploymentParser.T__12)
            self.state = 204
            self.match(deploymentParser.T__31)
            self.state = 205
            self.match(deploymentParser.T__11)
            self.state = 206
            self.match(deploymentParser.INT)
            self.state = 207
            self.match(deploymentParser.T__12)
            self.state = 208
            self.match(deploymentParser.T__32)
            self.state = 209
            self.match(deploymentParser.T__11)
            self.state = 210
            self.provider()
            self.state = 211
            self.match(deploymentParser.T__12)
            self.state = 212
            self.match(deploymentParser.T__33)
            self.state = 213
            self.match(deploymentParser.T__11)
            self.state = 214
            self.match(deploymentParser.STRING)
            self.state = 215
            self.match(deploymentParser.T__12)
            self.state = 216
            self.service_list()
            self.state = 217
            self.match(deploymentParser.T__12)
            self.state = 218
            self.deployment_list()
            self.state = 219
            self.match(deploymentParser.T__12)
            self.state = 220
            self.region_list()
            self.state = 225
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,12,self._ctx)
            if la_ == 1:
                self.state = 221
                self.match(deploymentParser.T__12)
                self.state = 222
                self.match(deploymentParser.T__34)
                self.state = 223
                self.match(deploymentParser.T__11)
                self.state = 224
                self.boolean()


            self.state = 237
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,14,self._ctx)
            if la_ == 1:
                self.state = 227
                self.match(deploymentParser.T__12)
                self.state = 228
                self.match(deploymentParser.T__35)
                self.state = 229
                self.match(deploymentParser.T__11)
                self.state = 230
                self.match(deploymentParser.T__27)
                self.state = 231
                self.match(deploymentParser.ID)
                self.state = 234
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==13:
                    self.state = 232
                    self.match(deploymentParser.T__12)
                    self.state = 233
                    self.match(deploymentParser.ID)


                self.state = 236
                self.match(deploymentParser.T__28)


            self.state = 249
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 239
                self.match(deploymentParser.T__12)
                self.state = 240
                self.match(deploymentParser.T__36)
                self.state = 241
                self.match(deploymentParser.T__11)
                self.state = 242
                self.match(deploymentParser.T__27)
                self.state = 243
                self.match(deploymentParser.ID)
                self.state = 246
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==13:
                    self.state = 244
                    self.match(deploymentParser.T__12)
                    self.state = 245
                    self.match(deploymentParser.ID)


                self.state = 248
                self.match(deploymentParser.T__28)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PrivateClusterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self):
            return self.getToken(deploymentParser.ID, 0)

        def getRuleIndex(self):
            return deploymentParser.RULE_privateCluster

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPrivateCluster" ):
                listener.enterPrivateCluster(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPrivateCluster" ):
                listener.exitPrivateCluster(self)




    def privateCluster(self):

        localctx = deploymentParser.PrivateClusterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_privateCluster)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 251
            self.match(deploymentParser.T__37)
            self.state = 252
            self.match(deploymentParser.T__1)
            self.state = 253
            self.match(deploymentParser.T__10)
            self.state = 254
            self.match(deploymentParser.T__11)
            self.state = 255
            self.match(deploymentParser.ID)
            self.state = 256
            self.match(deploymentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Service_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_service_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterService_list" ):
                listener.enterService_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitService_list" ):
                listener.exitService_list(self)




    def service_list(self):

        localctx = deploymentParser.Service_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_service_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 258
            self.match(deploymentParser.T__4)
            self.state = 259
            self.match(deploymentParser.T__11)
            self.state = 260
            self.match(deploymentParser.T__27)
            self.state = 261
            self.match(deploymentParser.ID)
            self.state = 264
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 262
                self.match(deploymentParser.T__12)
                self.state = 263
                self.match(deploymentParser.ID)


            self.state = 266
            self.match(deploymentParser.T__28)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Deployment_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID)
            else:
                return self.getToken(deploymentParser.ID, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_deployment_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDeployment_list" ):
                listener.enterDeployment_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDeployment_list" ):
                listener.exitDeployment_list(self)




    def deployment_list(self):

        localctx = deploymentParser.Deployment_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_deployment_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 268
            self.match(deploymentParser.T__6)
            self.state = 269
            self.match(deploymentParser.T__11)
            self.state = 270
            self.match(deploymentParser.T__27)
            self.state = 271
            self.match(deploymentParser.ID)
            self.state = 274
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 272
                self.match(deploymentParser.T__12)
                self.state = 273
                self.match(deploymentParser.ID)


            self.state = 276
            self.match(deploymentParser.T__28)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Region_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ID_REG(self, i:int=None):
            if i is None:
                return self.getTokens(deploymentParser.ID_REG)
            else:
                return self.getToken(deploymentParser.ID_REG, i)

        def getRuleIndex(self):
            return deploymentParser.RULE_region_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRegion_list" ):
                listener.enterRegion_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRegion_list" ):
                listener.exitRegion_list(self)




    def region_list(self):

        localctx = deploymentParser.Region_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_region_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 278
            self.match(deploymentParser.T__7)
            self.state = 279
            self.match(deploymentParser.T__11)
            self.state = 280
            self.match(deploymentParser.T__27)
            self.state = 281
            self.match(deploymentParser.ID_REG)
            self.state = 284
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==13:
                self.state = 282
                self.match(deploymentParser.T__12)
                self.state = 283
                self.match(deploymentParser.ID_REG)


            self.state = 286
            self.match(deploymentParser.T__28)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ProtocolContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return deploymentParser.RULE_protocol

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProtocol" ):
                listener.enterProtocol(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProtocol" ):
                listener.exitProtocol(self)




    def protocol(self):

        localctx = deploymentParser.ProtocolContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_protocol)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 288
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 17042430230528) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Service_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return deploymentParser.RULE_service_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterService_type" ):
                listener.enterService_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitService_type" ):
                listener.exitService_type(self)




    def service_type(self):

        localctx = deploymentParser.Service_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_service_type)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 290
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 123145302310912) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ProviderContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return deploymentParser.RULE_provider

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProvider" ):
                listener.enterProvider(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProvider" ):
                listener.exitProvider(self)




    def provider(self):

        localctx = deploymentParser.ProviderContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_provider)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 292
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 2111062325329920) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BooleanContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return deploymentParser.RULE_boolean

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBoolean" ):
                listener.enterBoolean(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBoolean" ):
                listener.exitBoolean(self)




    def boolean(self):

        localctx = deploymentParser.BooleanContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_boolean)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 294
            _la = self._input.LA(1)
            if not(_la==51 or _la==52):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








from antlr4 import CommonTokenStream, FileStream, ParseTreeWalker
from besser.utilities.buml_code_builder import domain_model_to_code
from besser.BUML.metamodel.structural import DomainModel
from .PlantUMLLexer import PlantUMLLexer
from .PlantUMLParser import PlantUMLParser
from .plantUML_buml_listener import BUMLGenerationListener

def plantuml_to_buml(plantUML_model_path:str, buml_file_path:str = None):
    """Transforms a PlantUML model into a B-UML model.

    Args:
        plantUML_model_path (str): The path to the file containing the PlantUML code.
        buml_file_path (str, optional): the path of the file produced with the base 
                code to build the B-UML model (None as default).

    Returns:
        BUML_model (DomainModel): the B-UML model object.
    """
    lexer = PlantUMLLexer(FileStream(plantUML_model_path))
    parser = PlantUMLParser(CommonTokenStream(lexer))
    parse_tree = parser.domainModel()
    listen = BUMLGenerationListener()
    walker = ParseTreeWalker()
    walker.walk(listen, parse_tree)
    domain_model: DomainModel = listen.get_buml_model()
    if buml_file_path is not None:
        domain_model_to_code(model=domain_model, file_path=buml_file_path)
    return domain_model



"""
This module provides the `SetupLayerSyntax` class and the 
`get_tensorop_syntax` function.
The `SetupLayerSyntax` class is used to define the syntax 
of layers in PyTorch, while `get_tensorop_syntax` defines the 
tensorOps.
"""

from besser.BUML.metamodel.nn import TensorOp
from besser.generators.nn import utils_nn as utils


class SetupLayerSyntax:
    """
    This class is used to map the BUML synatx for layers to PyTorch 
    syntax.
    It processes the layers based on their type.
    """
    def __init__(self, layer, modules_details):
        self.layer = layer
        self.modules_details: dict = modules_details
        self.permute_out: bool = None
        self.permute_in: bool = None
        self.dim: str = None

    def setup_general_layer(self):
        """It defines the syntax of general layers."""
        cls_name = self.layer.__class__.__name__
        lyr_name = self.layer.name
        lyr = f"self.{lyr_name} = nn"

        if cls_name == "LinearLayer":
            in_f = self.layer.in_features
            out_f = self.layer.out_features
            lyr = f"{lyr}.Linear(in_features={in_f}, out_features={out_f})"
        elif cls_name == "FlattenLayer":
            st_dim = self.layer.start_dim
            en_dim = self.layer.end_dim
            lyr = f"{lyr}.Flatten(start_dim={st_dim}, end_dim={en_dim})"
        else: #cls_name == "EmbeddingLayer"
            nm = self.layer.num_embeddings
            dm = self.layer.embedding_dim
            lyr = f"{lyr}.Embedding(num_embeddings={nm}, embedding_dim={dm})"

        return lyr

    def setup_layer_modifier(self):
        """It defines the syntax of layers' modifiers."""
        cls_name = self.layer.__class__.__name__
        parent_cls = self.layer.__class__.mro()[1].__name__
        lyr_name = self.layer.name
        lyr = f"self.{lyr_name} = nn"
        if parent_cls == "NormalizationLayer":
            if cls_name == "BatchNormLayer":
                dim = self.layer.dimension[0]
                num_f = self.layer.num_features
                lyr = f"{lyr}.BatchNorm(){dim}d(num_features={num_f})"
            else: #cls_name == "LayerNormLayer"
                norm_shape = self.layer.normalized_shape
                if norm_shape[1] is not None:
                    if norm_shape[2] is not None:
                        shape = [norm_shape[0], norm_shape[1], norm_shape[2]]
                    else:
                        shape = [norm_shape[0], norm_shape[1]]
                else:
                    shape = [norm_shape[0]]

                lyr = f"{lyr}.LayerNorm(normalized_shape={shape})"
        else: #cls_name == "DropoutLayer"
            lyr = f"{lyr}.Dropout(p={self.layer.rate})"
        return lyr

    def add_permute(self, lyr_name, dim, in_var_layer, permute_in = True,
                    sequential=False, is_subnn=False):
        """It permutes the input of the layer"""
        if permute_in:
            perm_name = f"{lyr_name}_in_op"
        else:
            perm_name = f"{lyr_name}_out_op"
        if dim is None:
            perm_dim = [0, 2, 1]
        else:
            if dim == "1":
                perm_dim = [0, 2, 1]
            elif dim == "2":
                if permute_in:
                    perm_dim = [0, 3, 1, 2]
                else:
                    perm_dim = [0, 2, 3, 1]
            else:
                if permute_in:
                    perm_dim = [0, 4, 1, 2, 3]
                else:
                    perm_dim = [0, 2, 3, 4, 1]

        if sequential or is_subnn:
            self.modules_details[perm_name] = [f"Permute(dims={perm_dim})",
                                               in_var_layer]
        else:
            tns = TensorOp(name=perm_name, tns_type="permute",
                           permute_dim=perm_dim)
            tns_out = utils.handle_tensorop
            self.modules_details = tns_out(tns, self.modules_details,
                                           get_tensorop_syntax,
                                           in_var_layer)


    def setup_rnn(self):
        """It defines the syntax of rnn layers."""
        cls_name = self.layer.__class__.__name__
        lyr_name = self.layer.name
        layer_type = cls_name[:-5]
        in_sz = self.layer.input_size
        h_sz = self.layer.hidden_size
        bd = self.layer.bidirectional
        drp = self.layer.dropout
        btch = self.layer.batch_first
        lyr = (
            f"self.{lyr_name} = nn.{layer_type}(input_size={in_sz}, "
            f"hidden_size={h_sz}, bidirectional={bd}, dropout={drp}, "
            f"batch_first={btch})"
        )
        return lyr, self.modules_details


    def setup_actv_func(self):
        """It defines the syntax of activation functions."""
        lyr = None
        activs = {"relu": "ReLU", "leaky_relu": "LeakyReLU",
                  "sigmoid": "Sigmoid", "softmax": "Softmax", "tanh": "Tanh"}
        if hasattr(self.layer, 'actv_func'):
            actv = self.layer.actv_func
            if actv in activs:
                lyr = f"self.actv_func_{actv} = nn.{activs[actv]}()"
            elif actv is not None:
                if actv.startswith("self"):
                    lyr = f"self.actv_func_{actv[5:]}"
                else:
                    lyr = f"self.actv_func_{actv}"
                lyr = f"{lyr} = get_activation_function({actv})"
        return lyr

    def setup_cnn(self):
        """It defines the syntax of cnn layers (conv and pooling)."""
        cls_name = self.layer.__class__.__name__
        lyr_name = self.layer.name

        if cls_name == "PoolingLayer":
            lyr = self.setup_pooling(lyr_name)
        else:
            lyr = self.setup_conv(lyr_name, cls_name)

        #self.modules_details is returned to have the same structure
        #of setup_rnn from torch
        return lyr, self.modules_details

    def setup_conv(self, lyr_name, cls_name):
        """It defines the syntax of convolutional layers."""
        dim = cls_name[-2:-1]
        in_chan = self.layer.in_channels
        out_chan = self.layer.out_channels
        kernel = utils.format_value(self.layer.kernel_dim)
        stride = utils.format_value(self.layer.stride_dim)
        pad = self.layer.padding_amount
        self.permute_in = self.layer.permute_in
        self.permute_out = self.layer.permute_out
        self.dim = dim
        lyr = (
            f"self.{lyr_name} = nn.Conv{dim}d(in_channels={in_chan}, "
            f"out_channels={out_chan}, kernel_size={kernel}, "
            f"stride={stride}, padding={pad})"
        )
        return lyr


    def setup_pooling(self, lyr_name):
        """It defines the syntax of pooling layers."""
        pl_type = self.layer.pooling_type
        dim = self.layer.dimension[-2:-1]
        self.dim = dim

        self.permute_in = self.layer.permute_in
        self.permute_out = self.layer.permute_out

        if pl_type == "max" or pl_type == "average":
            pl = "Max" if pl_type == "max" else "Avg"
            kernel = utils.format_value(self.layer.kernel_dim)
            stride = utils.format_value(self.layer.stride_dim)
            pad = self.layer.padding_amount
            lyr = (
                f"self.{lyr_name} = nn.{pl}Pool{dim}d(kernel_size={kernel}, "
                f"stride={stride}, padding={pad})"
            )
        elif pl_type.startswith("global"):
            out_dim = (1,) * int(dim)
            lyr = (
                f"self.{lyr_name} = nn.AdaptiveAvgPool{dim}d({out_dim})"
            )
            # or tensor.mean(dim=(2, 3, 4), keepdim=True)
        else:
            if pl_type == "adaptive_average":
                pl = "AdaptiveAvg"
            else:
                pl = "AdaptiveMax"

            size = utils.format_value(self.layer.output_dim)
            lyr = (
                f"self.{lyr_name} = nn.{pl}Pool{dim}d(output_size={size})"
            )
        return lyr

def get_tensorop_syntax(tensorop, modules_details, in_var=None):
    """It defines the syntax of tensorops."""

    prev_out_var, params = utils.get_tensorop_params(tensorop,
                                                     modules_details)
    if in_var is not None:
        prev_out_var = in_var

    tns_type = tensorop.tns_type
    if tns_type == "reshape":
        ts_op_synt = f"{prev_out_var}.reshape({params})"
    elif tns_type == "concatenate":
        dim = tensorop.concatenate_dim
        ts_op_synt = f"torch.cat(({params}), dim={dim})"
    elif tns_type == "transpose":
        ts_op_synt = f"{prev_out_var}.transpose({params})"
    elif tns_type == "permute":
        ts_op_synt = f"{prev_out_var}.permute({params})"
    elif tns_type == "multiply":
        ts_op_synt = f"torch.mul({params})"
    else:
        ts_op_synt = f"torch.matmul({params})"
    return ts_op_synt


def adjust_actv_func_name(modules_details):
    """Renames activation functions as activ_func_1, activ_func_2, ..."""
    actv_dict = {}
    counter = 1
    for mdl_name, mdl_details in modules_details.items():
        if mdl_name.split("_")[-1] == "activ":
            synt = mdl_details[0]
            if "get_activation_function" in synt:
                activ_type = synt.split("(")[1].split(")")[0]
                if activ_type not in actv_dict:
                    actv_dict[activ_type] = f"activ_func_{counter}"
                    counter+=1
                activ_def = synt.split("=")[1]
                mdl_details[0] = f"self.{actv_dict[activ_type]} = {activ_def}"
    return modules_details


def get_activation_function(activ):
    """
    It returns the activation function syntax if the user does not
    explicitely provide the activation function name in the BUML model.
    """
    activ_func = {"relu": "ReLU", "leaky_relu": "LeakyReLU",
                  "sigmoid": "Sigmoid", "softmax": "Softmax", "tanh": "Tanh"}
    activ = activ.lower()

    if activ in activ_func:
        return f"nn.{activ_func[activ]}()"
    raise ValueError(f"The activation function {activ} is invalid")